// $Id: CaptureViewFrame.java,v 1.8 2001/06/05 07:16:48 pcharles Exp $

/***************************************************************************
 * Copyright (C) 2001, Patrick Charles and Jonas Lehmann                   *
 * Distributed under the Mozilla Public License                            *
 *   http://www.mozilla.org/NPL/MPL-1.1.txt                                *
 ***************************************************************************/
package net.sourceforge.jpcap.client;

import java.util.Iterator;
import java.util.HashMap;
import javax.swing.JFrame;
import net.sourceforge.jpcap.net.IPPort;
import net.sourceforge.jpcap.net.IPProtocol;
import net.sourceforge.jpcap.net.Packet;
import net.sourceforge.jpcap.net.IPPacket;
import net.sourceforge.jpcap.net.TCPPacket;
import net.sourceforge.jpcap.net.UDPPacket;
import net.sourceforge.jpcap.net.ICMPPacket;
import net.sourceforge.jpcap.net.ICMPMessage;


/**
 * A graphical view of captured packets.
 *
 * @author Patrick Charles and Jonas Lehmann
 * @version $Revision: 1.8 $
 * @lastModifiedBy $Author: pcharles $
 * @lastModifiedAt $Date: 2001/06/05 07:16:48 $
 */
public class CaptureViewFrame extends JFrame 
{
  /**
   * Create a new capture view frame.
   */
  public CaptureViewFrame() {
    super("Network Packet Capture Frame");

    pvc = new PacketVisualizationCanvas(hostMap, commMap);
    this.setSize(Settings.DEFAULT_CANVAS_X, Settings.DEFAULT_CANVAS_Y);
    this.getContentPane().add(pvc);

    // to keep the view frame simple, delegate mouse event handling..
    CaptureViewMouseListener cvml = new CaptureViewMouseListener(pvc);
    pvc.addMouseListener(cvml);
    pvc.addMouseMotionListener(cvml);
  }

  /**
   * If a renderer doesn't exist for a host, create a new one and add 
   * it to the renderer map. Otherwise, simply fetch the existing renderer.
   * @param ipp the IP protocol packet
   * @param srcFlag whether the desired host is the source or destination
   * of the specified packet.
   * @return a host renderer
   */
  private HostRenderer addHostRenderer(IPPacket ipp, boolean srcFlag) {
    String a = srcFlag ? ipp.getSourceAddress() : ipp.getDestinationAddress();
    HostRenderer hr;
    if(hostMap.keySet().contains(a))
      hr = (HostRenderer)hostMap.get(a);
    else {
      hr = new HostRenderer(pvc, a);
      hostMap.put(a, hr);
    }

    return hr;
  }

  /**
   * Update a single packet in the view.
   * <p>
   * When a packet arrives, three renderers are created (or, if they 
   * already exist, are pulled from cache.) There are two renderers 
   * to represent the source and destination hosts and a communication 
   * renderer for the protocol via which the hosts are communicating.
   */
  public void update(Packet packet) {
    if(packet instanceof IPPacket) {
      HostRenderer src = addHostRenderer((IPPacket)packet, true);
      HostRenderer dst = addHostRenderer((IPPacket)packet, false);

      int srcPort = IPPort.NONE;
      int dstPort = IPPort.NONE;
      int byteCount = 0;
      if(packet instanceof UDPPacket) {
        srcPort = ((UDPPacket)packet).getSourcePort();
        dstPort = ((UDPPacket)packet).getDestinationPort();
        byteCount += ((UDPPacket)packet).getData().length;
      }
      else if(packet instanceof TCPPacket) {
        srcPort = ((TCPPacket)packet).getSourcePort();
        dstPort = ((TCPPacket)packet).getDestinationPort();
        byteCount += ((TCPPacket)packet).getData().length;
      }
      else if(packet instanceof ICMPPacket) {
        // icmp doesn't have ports, use message type instead..
        srcPort = dstPort = ((ICMPPacket)packet).getMessageType();
        byteCount += ((ICMPPacket)packet).getData().length;
      }
      else {
        // must be unrecognized IPPacket..
        srcPort = dstPort = ((IPPacket)packet).getProtocol();
        byteCount += ((IPPacket)packet).getData().length;
      }


      String description = "unknown";
      CommRenderer commRenderer;
      int keyPort = CommRenderer.keyPort(srcPort, dstPort);
      String commKey = CommRenderer.commKey(src.getIpAddress(), 
                                            dst.getIpAddress(), keyPort);
      if(commMap.keySet().contains(commKey)) {
        if(Settings.SHOW_COUNTS) {
          // if the renderer is already in the collection, just update stats..
          commRenderer = (CommRenderer)commMap.get(commKey);
          // the erase/paint() is broken because of a flaw in the 'slots' hack
          //        commRenderer.erase();
          commRenderer.incrementConnectionCount();
          commRenderer.incrementByteCount(byteCount);
          //        commRenderer.paint();
        }
      }
      else {
        if(packet instanceof TCPPacket || packet instanceof UDPPacket)
          // tcp or udp? label with service name or port number
          description = IPPort.getName(keyPort);
        else if(packet instanceof ICMPPacket)
          // icmp? label with icmp message type
          description = ICMPMessage.getDescription(srcPort);
        else 
          // unidentified ip? label with IP protocol name
          description = IPProtocol.getDescription(srcPort);
          
        commRenderer = new CommRenderer(pvc, packet, src, dst, description);
        commRenderer.incrementByteCount(byteCount);
        commMap.put(commKey, commRenderer);
      }
    }
  }

  /**
   * Update all packets in the view.
   * Iterate through all captured packets, creating renderers for all 
   * hosts and communications. This operation is expensive. Should never 
   * need to be called except when creating the UI for the first time.
   */
  public void refresh(CaptureHistory history) {
    Iterator iterator = history.getCollection().iterator(); 
    while(iterator.hasNext()) {
      Packet packet = (Packet)iterator.next();
      update(packet);
    }
  }

  /**
   * Reset the view frame. Removes all existing renderers, and regenerates
   * the network view from scratch.
   */
  public void reset(CaptureHistory history) {
    hostMap.clear();
    commMap.clear();

    refresh(history);
  }


  /**
   * Host renderer cache.
   */
  HashMap hostMap = new HashMap();

  /**
   * Communication renderer cache.
   */
  HashMap commMap = new HashMap();

  /**
   * Canvas where network packets are being rendered.
   */
  PacketVisualizationCanvas pvc;

  private String _rcsid = 
    "$Id: CaptureViewFrame.java,v 1.8 2001/06/05 07:16:48 pcharles Exp $";
}
